from ortools.sat.python import cp_model
from data import create_data
from solution_printer import NursesPartialSolutionPrinter

'''
A hospital supervisor needs to create a schedule for four nurses over a three-day period, subject to the following conditions:
  - Each day is divided into three 8-hour shifts.
  - Every day, each shift is assigned to a single nurse, and no nurse works more than one shift.
  - Each nurse is assigned to at least two shifts during the three-day period.
'''

def main():
    # Create data
    num_nurses, num_shifts, num_days, all_nurses, all_shifts, all_days, shift_requests = create_data()

    # Create the model
    model = cp_model.CpModel()

    # Create shift variables
    # shifts[(n, d, s)]: nurse 'n' works shift 's' on day 'd'.
    shifts = {}
    for n in all_nurses:
        for d in all_days:
            for s in all_shifts:
                shifts[(n, d, s)] = model.NewBoolVar('shift_n%id%is%i' % (n, d, s))

    # Each shift is assigned to exactly one nurse in the schedule period
    for d in all_days:
        for s in all_shifts:
            model.Add(sum(shifts[(n, d, s)] for n in all_nurses) == 1)

    # Each nurse works at most one shift per day.
    for n in all_nurses:
        for d in all_days:
            model.Add(sum(shifts[(n, d, s)] for s in all_shifts) <= 1)

    # Try to distribute the shifts evenly, so that each nurse works
    # min_shifts_per_nurse shifts. If this is not possible, because the total
    # number of shifts is not divisible by the number of nurses, some nurses will
    # be assigned one more shift
    min_shifts_per_nurse = (num_shifts * num_days) // num_nurses
    if num_shifts * num_days % num_nurses == 0:
        max_shifts_per_nurse = min_shifts_per_nurse
    else:
        max_shifts_per_nurse = min_shifts_per_nurse + 1
    for n in all_nurses:
        num_shifts_worked = 0
        for d in all_days:
            for s in all_shifts:
                num_shifts_worked += shifts[(n, d, s)]
        model.Add(min_shifts_per_nurse <= num_shifts_worked)
        model.Add(num_shifts_worked <= max_shifts_per_nurse)
    
    # Add objective function to maximize requested shifts
    model.Maximize(
        sum(shift_requests[n][d][s] * shifts[(n, d, s)] 
            for n in all_nurses for d in all_days for s in all_shifts))   

    # Creates the solver and solve
    solver = cp_model.CpSolver()

    # Display the first five solutions.
    a_few_solutions = range(5)
    output_file = open('Nurses-Partial-Solution-Printer.txt','a')
    solution_printer = NursesPartialSolutionPrinter(shifts, num_nurses,
                                                    num_days, num_shifts,
                                                    shift_requests, a_few_solutions,
                                                    output_file)
    solver.Solve(model, solution_printer)

    # Statistics
    print()
    print('Statistics')
    print('  - Number of shift requests met = %i' % solver.ObjectiveValue(),
        '(out of', num_nurses * min_shifts_per_nurse, ')')
    print('  - conflicts       : %i' % solver.NumConflicts())
    print('  - branches        : %i' % solver.NumBranches())
    print('  - wall time       : %f s' % solver.WallTime())
    print('  - solutions found : %i' % solution_printer.solution_count())

    output_file.write('\n')
    output_file.write('Statistics\n')
    output_file.write('  - Number of shift requests met = %i' % solver.ObjectiveValue() +
        '(out of %i' %(num_nurses * min_shifts_per_nurse) + ')\n')
    output_file.write('  - conflicts       : %i\n' % solver.NumConflicts())
    output_file.write('  - branches        : %i\n' % solver.NumBranches())
    output_file.write('  - wall time       : %f s\n' % solver.WallTime())
    output_file.write('  - solutions found : %i\n' % solution_printer.solution_count())

    output_file.close()


if __name__ == '__main__':
    main()