"""The Stigler diet problem.

A description of the problem can be found here:
https://en.wikipedia.org/wiki/Stigler_diet
"""
from os import write
from ortools.linear_solver import pywraplp
from data import create_data

def main():

    # Create output file
    output_file = open('Stigler-diet-problem-solution-printer.txt','a')
    
    # Instantiate the problem's data.
    nutrients, data = create_data()

    # Instantiate a Glop solver and naming it.
    solver = pywraplp.Solver('StiglerDietExample',
                             pywraplp.Solver.GLOP_LINEAR_PROGRAMMING)

    # Declare an array to hold our variables.
    foods = [solver.NumVar(0.0, solver.infinity(), item[0]) for item in data]

    print('Number of variables =', solver.NumVariables())
    output_file.write('Number of variables = {}\n'.format(solver.NumVariables()))

    # Create the constraints, one per nutrient.
    constraints = []
    for i, nutrient in enumerate(nutrients):
        constraints.append(solver.Constraint(nutrient[1], solver.infinity()))
        for j, item in enumerate(data):
            constraints[i].SetCoefficient(foods[j], item[i + 3])

    print('Number of constraints =', solver.NumConstraints())
    output_file.write('Number of constraints = {}\n'.format(solver.NumConstraints()))
    
    # Objective function: Minimize the sum of (price-normalized) foods.
    objective = solver.Objective()
    for food in foods:
        objective.SetCoefficient(food, 1)
    objective.SetMinimization()

    status = solver.Solve()

    # Check that the problem has an optimal solution.
    if status != solver.OPTIMAL:
        print('The problem does not have an optimal solution!')
        if status == solver.FEASIBLE:
            print('A potentially suboptimal solution was found.')
            output_file.write('A potentially suboptimal solution was found.\n')
        else:
            print('The solver could not solve the problem.')
            output_file.write('The solver could not solve the problem.\n')
            exit(1)

    # Display the amounts (in dollars) to purchase of each food.
    nutrients_result = [0] * len(nutrients)
    print('\nAnnual Foods:')
    output_file.write('\nAnnual Foods:\n')
    for i, food in enumerate(foods):
        if food.solution_value() > 0.0:
            print('{}: ${}'.format(data[i][0], 365. * food.solution_value()))
            output_file.write('{}: ${}\n'.format(data[i][0], 365. * food.solution_value()))
            for j, _ in enumerate(nutrients):
                nutrients_result[j] += data[i][j + 3] * food.solution_value()
    print('\nOptimal annual price: ${:.4f}'.format(365. * objective.Value()))
    output_file.write('\nOptimal annual price: ${:.4f}\n'.format(365. * objective.Value()))

    print('\nNutrients per day:')
    output_file.write('\nNutrients per day:\n')
    for i, nutrient in enumerate(nutrients):
        print('{}: {:.2f} (min {})'.format(nutrient[0], nutrients_result[i], nutrient[1]))
        output_file.write('{}: {:.2f} (min {})\n'.format(nutrient[0], nutrients_result[i], nutrient[1]))

    print('\nAdvanced usage:')
    print('Problem solved in ', solver.wall_time(), ' milliseconds')
    print('Problem solved in ', solver.iterations(), ' iterations')

    output_file.write('\nAdvanced usage:\n')
    output_file.write('Problem solved in {} milliseconds\n'.format(solver.wall_time()))
    output_file.write('Problem solved in {} iterations\n'.format(solver.iterations()))

    output_file.close()


if __name__ == '__main__':
    main()